'use strict';

exports.__esModule = true;
exports.delimiterBuffer = exports.delimiter = exports.NextionProtocol = exports.nextionProtocol = undefined;
exports.read = read;

var _binProtocol = require('bin-protocol');

var _codes = require('./codes');

var _fp = require('lodash/fp');

var _fp2 = _interopRequireDefault(_fp);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _util = require('./util');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var debug = (0, _debug2.default)('nextion:protocol');

var NextionProtocol = (0, _binProtocol.createProtocol)(function () {
  // this is mostly here for the convenience of a 3p consumer.
  // TODO: document
  var reset = this.reader.reset;
  this.reader.reset = function (buf) {
    if (buf.slice(buf.length - 3).equals(delimiterBuffer)) {
      return reset.call(this, buf.slice(0, -3));
    }
    return reset.call(this, buf);
  };
});

var readers = {
  byte(name) {
    this.UInt8(name);
  },
  eventCode() {
    this.byte('code');
    return _codes.eventCodeMap.get(String(this.context.code));
  },
  responseCode() {
    this.byte('code');
    return _codes.responseCodeMap.get(String(this.context.code));
  },
  touchEvent() {
    this.pageId().byte('buttonId').byte('releaseEvent');
    this.context.releaseEvent = Boolean(this.context.releaseEvent);
  },
  pageId() {
    this.byte('pageId');
  },
  touchCoordinate() {
    this.byte('xHigh').byte('xLow').byte('yHigh').byte('yLow').byte('releaseEvent');
    this.context.releaseEvent = Boolean(this.context.releaseEvent);
  },
  touchCoordinateOnWake() {
    this.touchCoordinate();
  },
  stringData() {
    this.context.value = this.buffer.toString();
  },
  numericData() {
    this.Int16LE('value');
  }
};

Object.keys(readers).forEach(function (name) {
  NextionProtocol.define(name, {
    read: readers[name]
  });
});

function read(data) {
  var code = data.readUInt8(0);
  debug('Code:', (0, _util.hexStr)(code));
  var name = _codes.responseCodeMap.get(String(code));
  if (name) {
    return {
      name,
      codeByte: (0, _util.hexStr)(code),
      code,
      type: 'response'
    };
  }
  name = _codes.eventCodeMap.get(String(code));
  if (name) {
    var result = {
      name,
      codeByte: (0, _util.hexStr)(code),
      code,
      type: 'event'
    };
    var reader = nextionProtocol.read(data.slice(1));
    if (_fp2.default.isFunction(reader[name])) {
      result.data = reader[name]().result;
    }
    return result;
  }
  throw new Error(`Unknown data received: ${data.toString()}`);
}

var nextionProtocol = exports.nextionProtocol = new NextionProtocol();

exports.NextionProtocol = NextionProtocol;
var delimiter = exports.delimiter = [0xff, 0xff, 0xff];
var delimiterBuffer = exports.delimiterBuffer = Buffer.from(delimiter);
//# sourceMappingURL=protocol.js.map