'use strict';

exports.__esModule = true;
exports.System = exports.SYSTEM_VARIABLES = undefined;

var _fp = require('lodash/fp');

var _fp2 = _interopRequireDefault(_fp);

var _events = require('events');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var isUnsignedInteger = _fp2.default.overEvery([_fp2.default.isNumber, _fp2.default.isFinite, _fp2.default.gte(0)]);

var SYSTEM_VARIABLES = exports.SYSTEM_VARIABLES = ['sys0', 'sys1', 'sys2'];

var System = exports.System = function (_EventEmitter) {
  _inherits(System, _EventEmitter);

  function System(uart) {
    _classCallCheck(this, System);

    var _this = _possibleConstructorReturn(this, _EventEmitter.call(this));

    _this.uart = uart;
    return _this;
  }

  System.prototype.setSystemVariable = function setSystemVariable(name, value) {
    var _this2 = this;

    return Promise.resolve().then(function () {
      if (!_fp2.default.includes(name, SYSTEM_VARIABLES)) {
        throw new Error(`"name" must be one of: ${SYSTEM_VARIABLES.join(', ')}, but found "${name}"`);
      }
      if (!isUnsignedInteger(value)) {
        throw new Error('"value" must be an unsigned integer <= 4294967295');
      }
      return _this2.uart.setValue(name, value);
    });
  };

  System.prototype.setRandomRange = function setRandomRange(min, max) {
    var _this3 = this;

    return Promise.resolve().then(function () {
      if (!isUnsignedInteger(min)) {
        throw new Error('"min" must be an unsigned integer <= 4294967295');
      }
      if (!isUnsignedInteger(max)) {
        throw new Error('"max" must be an unsigned integer <= 4294967295');
      }

      return _this3.uart.request(`ranset ${min},${max}`);
    });
  };

  System.prototype.random = function random() {
    var _this4 = this;

    var min = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
    var max = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;

    return Promise.resolve(function () {
      if (max < min) {
        throw new Error('"max" cannot be less than "min"');
      }
      if (min < 0 || max < 0) {
        throw new Error('only positive integers allowed');
      }
      if (min > 0 || max > 0) {
        return _this4.setRandomRange(min, max);
      }
    }).then(function () {
      return _this4.uart.request('rand');
    });
  };

  System.prototype.setNoTouchSleepTimer = function setNoTouchSleepTimer() {
    var ms = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;

    return this.uart.setValue('thsp', Math.floor(ms / 1000));
  };

  System.prototype.sleep = function sleep() {
    return this.uart.setValue('sleep', true);
  };

  System.prototype.wake = function wake() {
    return this.uart.setValue('sleep', false);
  };

  return System;
}(_events.EventEmitter);
//# sourceMappingURL=system.js.map